/*
    control++ : System configuration tool
    Copyright (C) 2017-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_specialH
#define lcontrolpp_specialH

#include "libcontrol++.h"
#include "filesystem.h"

//-----------------------------------------------------------------------------
// Variable that can be assigned only once

template<typename T> class TWriteOnceVar{

    private:
        T var;
        bool flag = false;

    public:
        const T &Val(void) const{
            return var;
            }
        void operator=(const T &v){
            if(!flag){
                var = v;
                flag = true;
                }
            }

        TWriteOnceVar(const T &v) : var(v){}
        TWriteOnceVar(void){}
    };

//-----------------------------------------------------------------------------
// Set of params of a running program

class TRunMode{

    private:
        TWriteOnceVar<string> exe_path;
        TWriteOnceVar<bool> force;
        vector<TWriteOnceVar<bool>> flags;

    public:
        // Data access
        const string& ExePath(void) const{return exe_path.Val();}
        bool Force(void) const{return force.Val();}
        bool Flag(const int i) const{
            return (i < (int)flags.size()) ? flags.at(i).Val() : false;
            }
        // Modifiers
        void WriteExePath(const string &path){exe_path = FormatFilePath(path);}
        void WriteForce(const bool f){force = f;}
        void WriteFlag(const int i, const bool f);

        TRunMode(void) : force(false){}
    };

//-----------------------------------------------------------------------------

#endif
